use std::cmp::Ordering;

use super::api_error;
use super::FallibleApiResponse;
use super::ToAPI;
use mongod::Referencable;
use mongodb::bson::doc;
use rocket::fs::NamedFile;
use rocket::*;
use serde_json::json;

use crate::library::album::Album;
use crate::library::Libary;
use crate::route::to_api;

#[get("/artist/<artist_id>/albums")]
pub async fn albums_route(artist_id: &str, lib: &State<Libary>) -> FallibleApiResponse {
    let albums = lib.get_albums_by_artist(artist_id).await;

    Ok(json!({
        "artist": artist_id,
        "albums": to_api(&albums).await
    }))
}

fn sort_by_tracknumber(a: &serde_json::Value, b: &serde_json::Value) -> Ordering {
    a.get("tracknumber")
        .unwrap()
        .as_i64()
        .unwrap()
        .cmp(&b.get("tracknumber").unwrap().as_i64().unwrap())
}

#[get("/album/<album_id>/cover")]
pub async fn album_cover_route(album_id: &str, lib: &State<Libary>) -> Option<NamedFile> {
    let album = lib.get_album_by_id(album_id).await?;
    NamedFile::open(album.get_cover().await?).await.ok()
}

#[get("/album/<album_id>")]
pub async fn album_route(album_id: &str, lib: &State<Libary>) -> FallibleApiResponse {
    let album = lib
        .get_album_by_id(album_id)
        .await
        .ok_or_else(|| api_error("No album with that ID found"))?;

    let mut tracks = Album::get_tracks_of_album(&format!("album::{}", album._id))
        .await
        .into_iter()
        .map(|x| {
            json!({
                "id": x.id(),
                "title": x.title,
                "tracknumber": x.meta.map(|x| x.track_number())
            })
        })
        .collect::<Vec<_>>();

    tracks.sort_by(sort_by_tracknumber);

    let mut album = album.api().await;
    album
        .as_object_mut()
        .unwrap()
        .insert("tracks".into(), tracks.into());

    Ok(album)
}
